@@ -1,43 +1,61 @@
 from functools import wraps
 from time import sleep
 from types import MethodType
+from typing import TextIO
 
 from serial import serial_for_url
 from serial.serialutil import SerialException
+from yaml import FullLoader, dump, load
 
 from .extras import make_function
 from .io import read, read_byte_string, until, write
 from .protocol import parse_line
 
 
-_protocol = b'simpleRPC'
+_protocol = 'simpleRPC'
 _version = (3, 0, 0)
 
 _list_req = 0xff
 
 
+def _assert_protocol(protocol: str) -> None:
+    if protocol != _protocol:
+        raise ValueError('invalid protocol header')
+
+
+def _assert_version(version: tuple) -> None:
+    if version[0] != _version[0] or version[1] > _version[1]:
+        raise ValueError(
+            'version mismatch (device: {}, client: {})'.format(
+                '.'.join(map(str, version)),
+                '.'.join(map(str, _version))))
+
+
 class _Interface(object):
     """Generic simpleRPC interface."""
     def __init__(
             self: object, device: str, baudrate: int=9600, wait: int=2,
-            autoconnect: bool=True) -> None:
+            autoconnect: bool=True, load: TextIO=None) -> None:
         """
         :arg device: Device name.
         :arg baudrate: Baud rate.
         :arg wait: Time in seconds before communication starts.
         :arg autoconnect: Automatically connect.
+        :arg load: Load interface definition from file.
         """
         self._wait = wait
 
         self._connection = serial_for_url(
             device, do_not_open=True, baudrate=baudrate)
-        self._version = (0, 0, 0)
-        self._endianness = b'<'
-        self._size_t = b'H'
-        self.methods = {}
+        self.device = {
+            'endianness': '<',
+            'methods': {},
+            'protocol': '',
+            'size_t': 'H',
+            'version': (0, 0, 0)}
 
         if autoconnect:
-            self.open()
+            self.open(load)
 
     def __enter__(self: object) -> object:
         return self
@@ -60,86 +78,96 @@ class _Interface(object):
 
         :arg index: Method index.
         """
-        self._write(b'B', index)
+        self._write('B', index)
 
-    def _write(self: object, obj_type: bytes, obj: any) -> None:
+    def _write(self: object, obj_type: any, obj: any) -> None:
         """Provide parameters for a remote procedure call.
 
         :arg obj_type: Type of the parameter.
         :arg obj: Value of the parameter.
         """
-        write(self._connection, self._endianness, self._size_t, obj_type, obj)
+        write(
+            self._connection, self.device['endianness'], self.device['size_t'],
+            obj_type, obj)
 
     def _read_byte_string(self: object) -> bytes:
         return read_byte_string(self._connection)
 
-    def _read(self: object, obj_type: str) -> any:
+    def _read(self: object, obj_type: any) -> any:
         """Read a return value from a remote procedure call.
 
         :arg obj_type: Return type.
 
         :returns: Return value.
         """
-        return read(self._connection, self._endianness, self._size_t, obj_type)
+        return read(
+            self._connection, self.device['endianness'], self.device['size_t'],
+            obj_type)
 
-    def _get_methods(self: object) -> dict:
-        """Get remote procedure call methods.
-
-        :returns: Method objects indexed by name.
-        """
+    def _get_methods(self: object) -> None:
+        """Get remote procedure call methods."""
         self._select(_list_req)
 
-        if self._read_byte_string() != _protocol:
-            raise ValueError('missing protocol header')
+        _assert_protocol(self._read_byte_string().decode())
+        self.device['protocol'] = _protocol
 
-        self._version = tuple(self._read(b'B') for _ in range(3))
-        if self._version[0] != _version[0] or self._version[1] > _version[1]:
-            raise ValueError(
-                'version mismatch (device: {}, client: {})'.format(
-                    '.'.join(map(str, self._version)),
-                    '.'.join(map(str, _version))))
+        version = tuple(self._read('B') for _ in range(3))
+        _assert_version(version)
+        self.device['version'] = version
 
-        self._endianness, self._size_t = (
-            bytes([c]) for c in self._read_byte_string())
+        self.device['endianness'], self.device['size_t'] = (
+            chr(c) for c in self._read_byte_string())
 
-        methods = {}
         for index, line in enumerate(
                 until(lambda x: x == b'', self._read_byte_string)):
             method = parse_line(index, line)
-            methods[method['name']] = method
+            self.device['methods'][method['name']] = method
 
-        return methods
+    def _load(self: object, handle: TextIO=None) -> None:
+        """Load the interface definition from a file.
+
+        :arg handle: Open file handle.
+        """
+        self.device = load(handle, Loader=FullLoader)
+        _assert_protocol(self.device.get('protocol', ''))
+        _assert_version(self.device.get('version', (0, 0, 0)))
 
     def is_open(self: object) -> bool:
         """Query interface state."""
         pass
 
-    def open(self: object) -> None:
-        """Connect to device."""
+    def open(self: object, handle: TextIO=None) -> None:
+        """Connect to device.
+
+        :arg handle: Open file handle.
+        """
         sleep(self._wait)
 
-        self.methods = self._get_methods()
-        for method in self.methods.values():
+        if handle:
+            self._load(handle)
+        else:
+            self._get_methods()
+        for method in self.device['methods'].values():
             setattr(
                 self, method['name'], MethodType(make_function(method), self))
 
     def close(self: object) -> None:
         """Disconnect from device."""
-        for method in self.methods:
+        for method in self.device['methods']:
             delattr(self, method)
-        self.methods.clear()
+        self.device['methods'].clear()
 
     def call_method(self: object, name: str, *args: list) -> any:
         """Execute a method.
 
         :arg name: Method name.
-        :arg *args: Method parameters.
+        :arg args: Method parameters.
 
         :returns: Return value of the method.
         """
-        if name not in self.methods:
+        if name not in self.device['methods']:
             raise ValueError('invalid method name: {}'.format(name))
-        method = self.methods[name]
+        method = self.device['methods'][name]
 
         parameters = method['parameters']
         if len(args) != len(parameters):
@@ -160,6 +188,13 @@ class _Interface(object):
             return self._read(method['return']['fmt'])
         return None
 
+    def save(self: object, handle: TextIO) -> None:
+        """Save the interface definition to a file.
+
+        :arg handle: Open file handle.
+        """
+        dump(self.device, handle, width=76, default_flow_style=False)
+
 
 class SerialInterface(_Interface):
     """Serial simpleRPC interface."""
@@ -168,9 +203,9 @@ class SerialInterface(_Interface):
         return self._connection.isOpen()
 
     @wraps(_Interface.open)
-    def open(self: object) -> None:
+    def open(self: object, handle: TextIO=None) -> None:
         self._open()
-        super().open()
+        super().open(handle)
 
     @wraps(_Interface.close)
     def close(self: object) -> None:
@@ -195,7 +230,7 @@ class SocketInterface(_Interface):
 
     @wraps(_Interface.is_open)
     def is_open(self: object) -> bool:
-        return len(self.methods) > 0
+        return len(self.device['methods']) > 0
 
     open = _auto_open(_Interface.open)
     call_method = _auto_open(_Interface.call_method)
